-----------------------------
-- Get the addon table
-----------------------------

local AddonName = "iFriends";

local iFriends = LibStub("AceAddon-3.0"):GetAddon(AddonName);
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

---------------------------
-- Utility functions
---------------------------

local function strsplit(delimiter, text)
  local list = {}
  local pos = 1
  if strfind("", delimiter, 1) then -- this would result in endless loops
    --error("delimiter matches empty string!")
  end
  while 1 do
    local first, last = strfind(text, delimiter, pos)
    if first then -- found?
      tinsert(list, strsub(text, pos, first-1))
      pos = last+1
    else
      tinsert(list, strsub(text, pos))
      break
    end
  end
  return list
end

---------------------------------
-- The configuration table
---------------------------------

local function CreateConfig()
	CreateConfig = nil;
	
	local db = {
		type = "group",
		name = AddonName,
		order = 1,
		get = function(info)
			return iFriends.db.Column[info.arg.k][info.arg.v];
		end,
		set = function(info, value)
			iFriends.db.Column[info.arg.k][info.arg.v] = value;
		end,
		args = {
			Infotext1 = {
				type = "description",
				name = ("|cffff00ff%s |cffffffff%s|r |cffff00ff%s|r"):format(
					L["Write the columns in the order you wish to display into the following textbox. Example:"],
					"level, name", L["will display Level and Name columns."]
				),
				fontSize = "medium",
				order = 1,
			},
			Display = {
				type = "input",
				name = "",
				order = 2,
				width = "full",
				validate = function(info, value)
					local list = strsplit(",%s*", value);
					for i = 1, #list do
						if( not iFriends.Columns[list[i]] ) then
							return L["Invalid column name!"];
						end
					end
					return true;
				end,
				get = function(info)
					return table.concat(iFriends.db.Display, ", ");
				end,
				set = function(info, value)
					iFriends.db.Display = strsplit(",%s*", value);
					iFriends:GetDisplayedColumns();
				end,
			},
			Infotext2 = {
				type = "description",
				fontSize = "medium",
				order = 3,
			},
			--Sorting = {
			--	type = "select",
			--	name = L["Sorting"],
			--	order = 5,
			--	values = {
			--		["name"] = L["By Name"],
			--		["level"] = L["By Level"],
		--		},
			--	get = function() return iFriends.db.Sort end,
			--	set = function(info, value) iFriends.db.Sort = value end,
			--},
		},
	};
	
	local colnames = {};
	local i = 6;
	for k, v in pairs(iFriends.Columns) do
		table.insert(colnames, k);
		
		db.args["Header"..i] = {
			type = "header",
			name = L["Column"]..": "..v.Label,
			order = i
		};
		i = i + 1;
		
		db.args["ShowLabel"..i] = {
			type = "toggle",
			name = L["Show Label"],
			order = i,
			arg = {
				k = k,
				v = "ShowLabel",
			},
		};
		i = i + 1;
		
		db.args["Align"..i] = {
			type = "select",
			name = L["Align"],
			order = i,
			values = {
				["LEFT"] = L["Left"],
				["CENTER"] = L["Center"],
				["RIGHT"] = L["Right"],
			},
			arg = {
				k = k,
				v = "Align",
			},
		};
		i = i + 1;
	end
	
	db.args.Infotext2.name = ("|cffff00ff%s: |cffffffff%s|r |cffff00ff%s|r\n"):format(
		L["Available columns"],
		table.concat(colnames, ", "),
		L["- feel free to combine them like you want!"]
	);
	
	return db;
end

function iFriends:CreateDB()
	iFriends.CreateDB = nil;
	
	return { profile = {
		Display = { "realid", "realm", "level", "name", "race", "zone", "note" },
		--Sort = "name",
		Column = {
			realid = {
				ShowLabel = true,
				Align = "LEFT",
			},
			realm = {
				ShowLabel = true,
				Align = "CENTER",
			},
			level = {
				ShowLabel = false,
				Align = "RIGHT",
			},
			name = {
				ShowLabel = true,
				Align = "LEFT",
			},
			race = {
				ShowLabel = true,
				Align = "CENTER",
			},
			zone = {
				ShowLabel = true,
				Align = "CENTER",
			},
			note = {
				ShowLabel = true,
				Align = "LEFT",
			},
		},
	}};
end

function iFriends:OpenOptions()
	LibStub("AceConfigDialog-3.0"):Open(AddonName);
end

LibStub("AceConfig-3.0"):RegisterOptionsTable(AddonName, CreateConfig);
LibStub("AceConfigDialog-3.0"):AddToBlizOptions(AddonName);
LibStub("AceConfigDialog-3.0"):SetDefaultSize(AddonName, 400, 500);

_G.SlashCmdList["IFRIENDS"] = iFriends.OpenOptions;
_G["SLASH_IFRIENDS1"] = "/ifriends";