-----------------------------------
-- Setting up scope and libs
-----------------------------------

local AddonName = "iFriends"
iFriends = LibStub("AceAddon-3.0"):NewAddon(AddonName, "AceEvent-3.0", "AceTimer-3.0");

local QTip = LibStub("LibQTip-1.0");
local Tourist = LibStub("LibTourist-3.0");
local L = LibStub("AceLocale-3.0"):GetLocale(AddonName);

-----------------------------------------
-- Variables, functions and colors
-----------------------------------------

local Tooltip;

local Roster = {};

local ROSTER_BNET, ROSTER_GAME, ROSTER_CHAR_NAME, ROSTER_CHAR_LEVEL, ROSTER_CHAR_CLASS, ROSTER_CHAR_ZONE = 1, 2, 3, 4, 5, 6;
local ROSTER_CHAR_STATUS, ROSTER_CHAR_NOTE, ROSTER_CHAR_REALM, ROSTER_CHAR_RACE, ROSTER_CHAR_FACTION = 7, 8, 9, 10, 11;
local ROSTER_BNET_BROADCAST, ROSTER_BNET_REALID, ROSTER_BNET_PID, ROSTER_BNET_TOONID = 12, 13, 14, 15;

local NULL = "";

local COLOR_GOLD = "|cfffed100";
local COLOR_BNET = FRIENDS_BNET_NAME_COLOR_CODE;

local ClassColors = {};
for k, v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end
for k, v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	local c = RAID_CLASS_COLORS[k];
	ClassColors[v] = ("|cff%02x%02x%02x"):format(c.r *255, c.g *255, c.b *255);
end

local function tclear(t, wipe)
	if( type(t) ~= "table" ) then return end;
	for k in pairs(t) do
		t[k] = nil;
	end
	t[''] = 1;
	t[''] = nil;
	if( wipe ) then
		t = nil;
	end
end

-----------------------------
-- Setting up the feed
-----------------------------

local function Feed_OnClick(_, button)
	if( button == "LeftButton" ) then
		-- alt + left click = add new friend
		if( IsAltKeyDown() ) then
			if( BNFeaturesEnabled() ) then
				AddFriendEntryFrame_Collapse(true);
				AddFriendFrame.editFocus = AddFriendNameEditBox;
				StaticPopupSpecial_Show(AddFriendFrame);
				if( GetCVarBool("addFriendInfoShown") ) then
					AddFriendFrame_ShowEntry();
				else
					AddFriendFrame_ShowInfo();
				end
			else
				StaticPopup_Show("ADD_FRIEND");
			end
		-- left click = open friends frame
		else
			ToggleFriendsFrame(1);
		end
	-- right click will open iFriends options
	elseif( button == "RightButton" ) then
		iFriends:OpenOptions();
	end
end

local function Feed_OnEnter(anchor)
	if( #Roster < 1 ) then
		return;
	end
	
	for k, v in QTip:IterateTooltips() do
		if( type(k) == "string" and strsub(k, 1, 6) == "iSuite" ) then
			v:Release(k);
		end
	end
	
	ShowFriends();
	Tooltip = QTip:Acquire("iSuite"..AddonName);
	Tooltip:SetAutoHideDelay(0.1, anchor);
	Tooltip:SmartAnchorTo(anchor);
	iFriends:UpdateTooltip();
	Tooltip:Show();
end

iFriends.Feed = LibStub("LibDataBroker-1.1"):NewDataObject(AddonName, {
	type = "data source",
	text = "",
	icon = "Interface\\Addons\\iFriends\\Images\\iFriends",
	OnClick = Feed_OnClick,
	OnEnter = Feed_OnEnter,
});

-----------------
-- Columns
-----------------

local player_realm = GetRealmName();
iFriends.Columns = {
	realid = {
		Label = L["RealID"],
		Brush = function(v)
			return ("%s%s|r"):format(COLOR_BNET, v[ROSTER_BNET_REALID] or ("%s%s|r"):format(FRIENDS_OTHER_NAME_COLOR_CODE, L["local friend"]));--NULL);
		end,
	},
	realm = {
		Label = L["Game/Realm"],
		Brush = function(v)
			if( v[ROSTER_GAME] == BNET_CLIENT_WOW ) then
				if( v[ROSTER_BNET] ) then
					local factioncity = {L["Hordecity"], L["Allycity"]};
					local r, g, b = Tourist:GetFactionColor(factioncity[v[ROSTER_CHAR_FACTION] +1]);
					return ("|cff%02x%02x%02x%s|r"):format(r *255, g *255, b *255, v[ROSTER_CHAR_REALM]);
				else
					return ("|cff00ff00%s|r"):format(player_realm);
				end
			else
				if( v[ROSTER_GAME] == BNET_CLIENT_SC2 ) then
					return "Starcraft 2";
				elseif( v[ROSTER_GAME] == "D3" ) then
					return "Diablo 3";
				else
					return v[ROSTER_GAME];
				end
			end
		end,
	},
	level = {
		Label = L["Level"],
		Brush = function(v)
			if( v[ROSTER_GAME] == BNET_CLIENT_WOW ) then
				local c = GetQuestDifficultyColor(v[ROSTER_CHAR_LEVEL]);
				return ("|cff%02x%02x%02x%s|r"):format(c.r *255, c.g *255, c.b *255, v[ROSTER_CHAR_LEVEL]);
			else
				return NULL;
			end
		end,
	},
	name = {
		Label = L["Name"],
		Brush = function(v)
			if( v[ROSTER_GAME] == BNET_CLIENT_WOW ) then
				return ("%s%s%s|r"):format(ClassColors[v[ROSTER_CHAR_CLASS]], v[ROSTER_CHAR_STATUS] or NULL, v[ROSTER_CHAR_NAME]);
			else
				return NULL;
			end
		end,
		Script = function(_, v, button)
			if( button == "LeftButton" ) then
				SetItemRef(("player:%s"):format(v[ROSTER_CHAR_NAME]), ("|Hplayer:%s|h[%s]|h"):format(v[ROSTER_CHAR_NAME], v[ROSTER_CHAR_NAME]), "LeftButton");
			end
		end,
		ScriptUse = function(v) return v[ROSTER_BNET] and CanCooperateWithToon(v[ROSTER_BNET_TOONID]) end,
	},
	race = {
		Label = L["Race"],
		Brush = function(v)
			if( v[ROSTER_GAME] == BNET_CLIENT_WOW and v[ROSTER_BNET] ) then
				local factioncity = {L["Hordecity"], L["Allycity"]};
				local r, g, b = Tourist:GetFactionColor(factioncity[v[ROSTER_CHAR_FACTION] +1]);
				return ("|cff%02x%02x%02x%s|r"):format(r *255, g *255, b *255, v[ROSTER_CHAR_RACE]);
			else
				return NULL;
			end
		end,
	},
	zone = {
		Label = L["Zone"],
		Brush = function(v)
			if( v[ROSTER_GAME] == BNET_CLIENT_WOW ) then
				local r, g, b = Tourist:GetFactionColor(v[ROSTER_CHAR_ZONE]);
				return ("|cff%02x%02x%02x%s|r"):format(r *255, g *255, b *255, v[ROSTER_CHAR_ZONE]);
			else
				return NULL;
			end
		end,
	},
	note = {
		Label = L["Note"],
		Brush = function(v)
			if( v[ROSTER_BNET_BROADCAST] and v[ROSTER_BNET_BROADCAST] ~= "" ) then
				return ("%s%s|r"):format(COLOR_BNET, v[ROSTER_BNET_BROADCAST]);
			elseif( v.localNote and v.localNote ~= "" ) then
				return ("%s%s|r"):format(COLOR_GOLD, v[ROSTER_CHAR_NOTE]);
			end
			
			return NULL;
		end,
	},
};

local DisplayedColumns = {};
function iFriends:GetDisplayedColumns()
	tclear(DisplayedColumns);
	
	for i = 1, #self.db.Display do
		local insert = true;
		local canUse = self.Columns[self.db.Display[i]].CanUse;
		
		if( type(canUse) == "boolean" and not canUse ) then
			insert = false;
		elseif( type(canUse) == "function" and not canUse() ) then
			insert = false;
		end
		
		if( insert ) then
			table.insert(DisplayedColumns, self.db.Display[i]);
		end
	end
	
end

----------------------
-- OnInitialize
----------------------

function iFriends:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("iFriendsDB", self:CreateDB(), "Default").profile;
	
	self:GetDisplayedColumns();

	self:RegisterEvent("FRIENDLIST_UPDATE", "EventHandler");
	self:RegisterEvent("BN_FRIEND_INFO_CHANGED", "EventHandler");
	self:RegisterEvent("BN_FRIEND_LIST_SIZE_CHANGED", "EventHandler");
end

function iFriends:OnEnable()
	ShowFriends();
	self:ScheduleRepeatingTimer(ShowFriends, 55);
end

----------------------
-- EventHandler
----------------------

function iFriends:EventHandler()
	local friends, friendsOn = GetNumFriends();
	local bnet, bnetOn = 0, 0;
	
	if( BNFeaturesEnabledAndConnected() ) then
		bnet, bnetOn = BNGetNumFriends();
	end
	
	local total = friends + bnet;
	local totalOn = friendsOn + bnetOn;
	
	self.Feed.text = ("%d/%d"):format(totalOn, total);
	self:SetupFriendsData(friendsOn, bnetOn);
	
	if( QTip:IsAcquired("iSuite"..AddonName) ) then
		self:UpdateTooltip();
	end
end

--------------------------------------
-- SetupFriendsData and Sorting
--------------------------------------

local function Roster_Sort(a, b)
	if( not a or not b ) then
		return false;
	end
	
	if( a[ROSTER_BNET] and not b[ROSTER_BNET] ) then
		return false;
	elseif( not a[ROSTER_BNET] and b[ROSTER_BNET] ) then
		return true;
	else
		if( a[ROSTER_CHAR_LEVEL] < b[ROSTER_CHAR_LEVEL] ) then
			return true;
		elseif( a[ROSTER_CHAR_LEVEL] > b[ROSTER_CHAR_LEVEL] ) then
			return false;
		else
			if( a[ROSTER_CHAR_CLASS] < b[ROSTER_CHAR_CLASS] ) then
				return true;
			elseif( b[ROSTER_CHAR_CLASS] < b[ROSTER_CHAR_CLASS] ) then
				return false;
			else
				return a[ROSTER_CHAR_NAME] < b[ROSTER_CHAR_NAME];
			end
		end
	end
end

function iFriends:SetupFriendsData(friendsOn, bnetOn)
	tclear(Roster);
	
	-- iterate through our friends
	for i = 1, friendsOn do
		local charName, charLevel, charClass, charZone, isOnline, charStatus, charNote = GetFriendInfo(i);
		
		if( isOnline ) then
			Roster[i] = {};
			Roster[i][ROSTER_BNET] = false;
			Roster[i][ROSTER_GAME] = BNET_CLIENT_WOW;
			Roster[i][ROSTER_CHAR_NAME] = charName;
			Roster[i][ROSTER_CHAR_LEVEL] = charLevel;
			Roster[i][ROSTER_CHAR_CLASS] = charClass;
			Roster[i][ROSTER_CHAR_ZONE] = charZone;
			Roster[i][ROSTER_CHAR_STATUS] = charStatus or nil;
			Roster[i][ROSTER_CHAR_NOTE] = charNote or nil;
		end
	end
	
	-- iterate through our battle.net friends, if battle.net is connected
	if( BNFeaturesEnabledAndConnected() ) then
		for i = 1, bnetOn do
			local pID, givenName, surName, _, toonID, _, isOnline, _, isAFK, isDND, Broadcast, charNote, _, _ = BNGetFriendInfo(i);
			
			if( isOnline ) then
				local _, charName, Game, charRealm, _, charFaction, charRace, charClass, _, charZone, charLevel, _ = BNGetToonInfo(pID);
				local realID = BATTLENET_NAME_FORMAT:format(givenName, surName);
				
				local charStatus = nil;
				if( isAFK ) then
					charStatus = ("<%s>"):format(AFK);
				elseif( isDND ) then
					charStatus = ("<%s>"):format(DND);
				end
				
				Roster[friendsOn +i] = {};
				Roster[friendsOn +i][ROSTER_BNET] = true;
				Roster[friendsOn +i][ROSTER_GAME] = Game;
				Roster[friendsOn +i][ROSTER_CHAR_NAME] = charName;
				Roster[friendsOn +i][ROSTER_CHAR_LEVEL] = charLevel;
				Roster[friendsOn +i][ROSTER_CHAR_CLASS] = charClass;
				Roster[friendsOn +i][ROSTER_CHAR_ZONE] = charZone;
				Roster[friendsOn +i][ROSTER_CHAR_STATUS] = charStatus or nil;
				Roster[friendsOn +i][ROSTER_CHAR_NOTE] = charNote or nil;
				Roster[friendsOn +i][ROSTER_CHAR_REALM] = charRealm;
				Roster[friendsOn +i][ROSTER_CHAR_RACE] = charRace;
				Roster[friendsOn +i][ROSTER_CHAR_FACTION] = charFaction;
				Roster[friendsOn +i][ROSTER_BNET_BROADCAST] = Broadcast or nil;
				Roster[friendsOn +i][ROSTER_BNET_REALID] = realID;
				Roster[friendsOn +i][ROSTER_BNET_PID] = pID;
				Roster[friendsOn +i][ROSTER_BNET_TOONID] = toonID;
			end
		end
	end
	
	table.sort(Roster, Roster_Sort);
end

-----------------------
-- UpdateTooltip
-----------------------

local function iFriendsLineClick(_, v, button)
	if( IsAltKeyDown() ) then
		InviteUnit(v[ROSTER_CHAR_NAME]);
	else
		if( v[ROSTER_BNET] ) then
			local itemRef = ("%s:%d"):format(v[ROSTER_BNET_REALID], v[ROSTER_BNET_PID])
			SetItemRef(("BNplayer:%s"):format(itemRef), ("|HBNplayer:%s|h[%s]|h"):format(itemRef, v[ROSTER_BNET_REALID]), "LeftButton");
		else
			SetItemRef(("player:%s"):format(v[ROSTER_CHAR_NAME]), ("|Hplayer:%s|h[%s]|h"):format(v[ROSTER_CHAR_NAME], v[ROSTER_CHAR_NAME]), "LeftButton");
		end
	end
end

function iFriends:UpdateTooltip()
	Tooltip:Clear();
	Tooltip:SetColumnLayout(#DisplayedColumns);
	
	local name, info;
	
	for y = 0, #Roster do
		local member;
		
		for x = 1, #DisplayedColumns do
			name = DisplayedColumns[x];
			info = self.Columns[name];
			
			if( x == 1 ) then
				if( y == 0) then
					Tooltip:AddHeader(" ");
				else
					Tooltip:AddLine(" ");
				end
			end
			
			if( y == 0 ) then
				if( self.db.Column[name].ShowLabel ) then
					Tooltip:SetCell(y +1, x, info.Label, nil, self.db.Column[name].Align);
				end
				
			else
				member = Roster[y];
				Tooltip:SetCell(y +1, x, info.Brush(member), nil, self.db.Column[name].Align);
				
				if( info.Script and info.ScriptUse(member) ) then
					Tooltip:SetCellScript(y +1, x, "OnMouseDown", info.Script, member);
				end
			end
		end
		
		if( member ) then
			Tooltip:SetLineScript(y +1, "OnMouseDown", iFriendsLineClick, member);
			
			if( member[ROSTER_GAME] ~= BNET_CLIENT_WOW ) then
				local a, r, g, b = 0, 1, 1, 1;
				
				if( member[ROSTER_GAME] == BNET_CLIENT_SC2 ) then
					a, r, g, b = 0.6, 0.1, 0.8, 1;
				elseif( member[ROSTER_GAME] == "D3") then
					a, r, g, b = 0.6, 1, 0.1, 0.1;
				end
				
				Tooltip:SetLineColor(y +1, r, g, b, a);
			end
		end
	end
end